package avalon.categoryClassVioletParser;



import java.util.Arrays;
import java.util.HashMap;

public class ClassInterface {

    protected String id = "";
    protected String name = "";
    protected String classType = "";
    protected HashMap<String,Method> methods;
    protected int xPos = 0;
    protected int yPos = 0;

    public ClassInterface(String id, String classType) {
        this.id = id;
        this.classType = classType;

        methods = new HashMap<>();
    }

    public void setXPos(double xPos) {
        this.xPos = (int) xPos;
    }

    public void setYPos(double yPos) {
        this.yPos = (int) yPos;
    }

    public int getXPos() {
        return xPos;
    }

    public int getYPos() {
        return yPos;
    }

    public String getId() {
        return id;
    }

    public String getName() {
        return name;
    }

    public HashMap<String,Method> getMethods() {
        return methods;
    }

    public void setName(String name) {
        this.name = name.trim();
    }

    public void addMethod(String method) {
        // Step 1: trim the input -- if nothing is left, we're done
        method = method.trim();
        if (method.equals("")) {
            return;  // nothing to do
        }
        
        // Step 2: the format of a method decl/string is <type> <methodName>( <type>* ) = <qualifiedFileName>.<ext>
        //         let's split the string into 2 parts before the '=' and after
        
        String[] halves = method.split("=");
        if (halves.length != 2) {
            Main.errout("method declaration not in correct format : "+method);
            return;
        }
        String left = halves[0].trim();
        String right = halves[1].trim();

        // Step 3: parse the left half
        // Step 3.1: first check that there is at least 2 terms (<type> <method>()) is minimal
        String terms[] = left.split("[ (),]+");
        if (terms.length <2) {
            Main.errout("method declaration not in correct format : "+left);
            return;
        }
        if (methods.containsKey(terms[1])) {
            Main.errout("multiple methods with same name (" + terms[1] + ") in class " + name);
            return;
        }
        String mtype = terms[0];
        String mname = terms[1];
        String sigs[] = { };
        if (terms.length >2) {
            sigs = Arrays.copyOfRange(terms,2,terms.length);
        }
        
        // Step 3.2 now parse the right half -- it is a sequence of names terminated by an extension
        //          {<name>.}*<ext>
        String[] rterms = right.split("[ .]+");
        if (rterms.length <2) {
            Main.errout("method declaration not in correct format : "+right);
            return;
        }
        int limit = rterms.length -1;
        String ext = rterms[limit];
        String filename = rterms[0];
        String dot = ".";
        for (int i = 1; i<limit; i++) {
            filename = filename + dot + rterms[i];
        }
        // Method(String clas, String type, String name, String[] sigs, String fileName, String ext) 
        Method m = new Method(id,mtype,mname,sigs,filename,ext);
        methods.put(terms[1], m);
    }

    @Override
    public String toString() {
        String str = "";
        str += "Id: " + id
                + ", Name: " + name
                + ", ClassType: " + classType
                + ", x: " + xPos
                + ", y: " + yPos;

        str += ", Methods:";
        for (String method : methods.keySet()) {
            Method m = methods.get(method);
            str += m + ";";
        }

        return str;
    }

    public String getClassType() {
        return classType;
    }
}
