/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package avalon.tometamodel;

import PrologDB.Table;
import PrologDB.Tuple;
import CoreMDELite.Error;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;
import org.apache.commons.io.FileUtils;
import static avalon.tometamodel.common.db;

/**
 *
 * @author don
 */
public class GenDomainClasses extends common {

    static public boolean init = true;
    static public PrintWriter out;

    public static void marquee() {
        System.out.println("Usage: tometamodel.GenDomainClasses <filename.meta.pl>");
        System.out.println("       directory <filename>/ will be created");
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String... args) {

        // Step 1: let's make sure we open everything properly
        if (args.length != 1) {
            marquee();
            return;
        }
        if (init) {
            commonstuff(args);
        }

        // schema is a map of (tableName,array-list-of-column-names)
        // tables is a map of (tableName, list-of-tuples) where a tuple is a map 
        // (columnName,value) pairs
        // Step 2: create a fresh directory for the application       
        freshDirectory("boot");
        freshDirectory("boot/src");
        freshDirectory("boot/src/" + packageName);

        // Step 3: now generate java file per domain
        //        below arguments: (name of prolog database), (velocity file))
        Table classTable = db.getTable("class");
        Table methodTable = db.getTable("method");

        for (Tuple cls : classTable.getTuples()) {
            //class[id,"name","domain","ext",temporary,x,y]

            PrintWriter out = setOut(cls.get("name") + ".java");
            GenClassFile(cls, out, classTable, methodTable);
            out.close();
//            System.err.println("closing "+cls.get("name") + ".java");
        }
    }

    private static void freshDirectory(String name) {
        File appDir = new File(name);
        if (appDir.exists()) {
            try {
                FileUtils.deleteDirectory(appDir);
            } catch (Exception e) {
                throw new Error("can't delete directory " + name + " : " + e.getMessage());
            }
        }
        appDir.mkdir();
    }

    static PrintWriter setOut(String filename) {
        String RealFileName = "boot/src/" + packageName + "/" + filename;
        try {
            out = new PrintWriter(new BufferedWriter(new FileWriter(RealFileName)));
        } catch (Exception e) {
            throw new Error("can't open " + filename);
        }
        return out;
    }

    static void GenClassFile(Tuple cls, PrintWriter out, Table classTable, Table methodTable) {
        GenFrontEnd(cls, out, methodTable);
        GenBackEnd(cls, out, classTable, methodTable);
    }

    static void GenFrontEnd(Tuple cls, PrintWriter out, Table methodTable) {
        String clsName = cls.get("name");

        out.format("package %s;\n\n", packageName);
        String which = "MDELiteObject";
        if (cls.get("ext").equals("pl")) {
            which = "PrologObject";
        }
        out.format("import CoreMDELite.%s;\n", which);
        out.format("import java.util.LinkedList;\n");
        out.format("import java.util.List;\n\n");

        out.format("public class %s extends %s {\n\n", clsName, which);

        out.format("    public static %s current = null;\n", clsName);
        out.format("    public static List<%s> array = new LinkedList<>();\n\n", clsName);

        out.format("    public String domainName() {\n");
        out.format("        return \"%s\";\n", cls.get("domain").toLowerCase());
        out.format("    }\n\n");

        out.format("    public String domainType() {\n");
        out.format("        return \"%s\";\n", cls.get("ext").toLowerCase());
        out.format("    }\n\n");

        out.format("    public %s(String filename) {\n", clsName);
        out.format("        super(filename);\n");
        out.format("        current = this;\n");
        out.format("        array.add(this);\n");
        if (cls.get("temporary").equals("true")) {
            out.format("        temporary();\n");
        }
        out.format("    }\n\n");

        // now see if a multi-parameter constructor is needed
        // this arises if 'sig' is non-empty
        if (false) {
            boolean needMultiParamConstructor = false;
            for (Tuple meth : methodTable.getTuples()) {
                if (meth.get("cid").equals(cls.get("id"))) {
                    // this is a method of the given class
                    if (meth.get("sig").equals("")) {
                        continue;
                    }
                    needMultiParamConstructor = true;
                    break;
                }
            }
            if (needMultiParamConstructor) {
                out.format("    public %s(String filename, String... files) {\n", clsName);
                out.format("        super(filename,files);\n");
                out.format("        current = this;\n");
                out.format("        array.add(this);\n");
                if (cls.get("temporary").equals("true")) {
                    out.format("        temporary();\n");
                }
                out.format("    }\n\n");
            }
        }
    }

    static void GenBackEnd(Tuple cls, PrintWriter out, Table classTable, Table methodTable) {
        String rt;
        Tuple resultTypeTuple;

        String id = cls.get("id");
        for (Tuple meth : methodTable.getTuples()) {
            if (meth.get("cid").equals(id)) {
                // this method belongs to this class
                switch (meth.get("ext")) {
                    case "vm":
                        rt = meth.get("resulttype");
                        out.format("    public %s %s() {\n", rt, meth.get("name"));
                        out.format("        %s result = new %s(fileName);\n", rt, rt);
                        out.format("        invokeVm2t(result, \"libvm/%s.vm\");\n", meth.get("filename"));

                        resultTypeTuple = classTable.findFirst("name", rt); //findTuple(classTable, "name", rt);
                        if (resultTypeTuple.get("ext").equals("pl")) {
                            out.format("        result.conform();\n");
                        }
                        out.format("        return result;\n");
                        out.format("    }\n\n");
                        break;

                    case "pl":
                        rt = meth.get("resulttype");
                        out.format("    public %s %s() {\n", rt, meth.get("name"));
                        out.format("        %s result = new %s(fileName);\n", rt, rt);

                        Tuple toClass = classTable.findFirst("name",rt);              //findTuple(classTable, "name", rt);
                        Tuple fromClass =  classTable.findFirst("id",id);             //findTuple(classTable, "id", id);
                        //fromTo("Violet", "SDB", result);
                        out.format("        fromTo(\"%s\",\"%s\",result.fullName);\n", fromClass.get("domain"), toClass.get("domain"));
                        resultTypeTuple = classTable.findFirst("name", rt);           //  findTuple(classTable, "name", rt);
                        if (resultTypeTuple.get("ext").equals("pl")) {
                            out.format("        result.conform();\n");
                        }
                        out.format("        return result;\n");
                        out.format("    }\n\n");
                        break;

                    case "java":
                        String params = "";
                        String argus = "";
                        String sig = meth.get("sig");
                        if (!sig.equals("")) {
                            int cnt = 0;
                            String comma = ",";
                            for (String s : sig.split(",")) {
                                if (!s.contains("[]")) {
                                    params += comma + s + " x" + cnt;
                                    argus += comma + " x" + cnt++ + ".fullName";
                                }
                            }
                            if (params.length() > 1) {
                                params = params.substring(1);  // one extra comma that we don't need
                            }
                        }
                        String type = meth.get("resulttype");
                        if (type.contains("[]")) {
                            type = type.replace("[]", "");
                            out.format("    public List<%s> %s(%s) {\n", type, meth.get("name"), params);
                            out.format("        %s.array = new LinkedList<>();\n", type);
                            out.format("        %s.main(fullName%s);\n", meth.get("filename"), params);
                            out.format("        return %s.array;\n", type);
                            out.format("    }\n\n");

                        } else {
                            out.format("    public %s %s(%s) {\n", type, meth.get("name"), params);
                            out.format("        %s result = new %s(fileName);\n", type, type);
                            out.format("        %s.main(fullName, result.fullName%s);\n", meth.get("filename"), argus);
                            Tuple tuple = classTable.findFirst("name",type);            //findTuple(classTable, "name", type);
                            if (tuple == null) {
                                throw new Error("reference to undefined class " + type);
                            }
                            if (tuple.get("ext").equals("pl")) {
                                out.format("        result.conform();\n");
                            }
                            out.format("        return result;\n");
                            out.format("    }\n\n");
                        }
                        break;
                    default:
                        throw new Error("unrecognizable extension type " + meth.get("ext"));
                }
            }
        }
        out.format("}\n");
    }
}
