/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package avalon.tometamodel;

import PrologDB.Table;
import PrologDB.Tuple;
import CoreMDELite.Error;
import CoreMDELite.PrologObject;
import java.io.File;
import java.io.PrintStream;
import org.apache.commons.io.FileUtils;

/**
 *
 * @author don
 */
public class GenLibs extends common {

    static public boolean init = true;

    static final String libReadMe = "boot/lib/ReadMe.txt";
    static final String libpl = "boot/libpl/";
    static final String libvm = "boot/libvm/";
    static final String srcdir = "boot/src/";

    static PrintStream out;

    public static void marquee() {
        System.out.println("Usage: tometamodel.GenLibs <filename.meta.pl>");
        System.out.println("       directory <filename>/ will be created");
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String... args) {
        String currentFile;

        // Step 1: let's make sure we open everything properly
        if (args.length != 1) {
            marquee();
            return;
        }
        if (init) {
            commonstuff(args);
        }

        // db is m map of (tableName,array-list-of-column-names)
        // tables is m map of (tableName, list-of-tuples) where m tuple is m map 
        // (columnName,value) pairs
        // Step 2: create required subdirectories
        try {
            File boot = new File("boot");
            if (!(boot.exists() && boot.isDirectory())) {
                FileUtils.deleteDirectory(boot);
                boot.mkdir();
            }
            makeSubDirectories("lib", "libpl", "libvm", "Docs");
        } catch (Exception e) {
            throw new Error("unable to create boot/lib or /libpl or /libvm directories; " + e.getMessage());
        }

        // Step 3: foreach prolog domain, create some files
        Table classTable = db.getTable("class");
        for (Tuple c : classTable.getTuples()) {

            if (c.get("ext").equals("pl")) {

                // Step 3.1 Create libReadMe/ read me file
                out = setOut(libReadMe);
                out.format("Place in this directory all jar files that your MDELite application references\n\n");
                out.close();

                // Step 3.2 now create some libpl files, 3 per prolog domain
                // Step 3.2.1 first file is <domain>.conform.pl
                String dn = c.get("domain");
                currentFile = libpl + dn + ".conform.pl";
                out = setOut(currentFile);
                out.format("/* file %s.conform.pl */\n\n", dn);
                out.format("run:-isError('please supply a real conformance file for domain ', %s).\n", dn);
                out.close();

                // Step 3.2.2 next file is <domain>.run.pl
                currentFile = libpl + dn + ".run.pl";
                out = setOut(currentFile);
                out.format("/* file %s.run.pl */\n\n", dn);
                out.format("run:-printDbase(%s).\n", dn);
                out.close();

                // Step 3.2.2 last file has already been created <domain>.schema.pl 
                //            by m previous transformation
                //            copy x into m libpl/x file
                currentFile = dn + ".schema.pl";
                out = setOut(libpl + currentFile);
                PrologObject.addFiletoPS(currentFile, out);
                out.close();

                // Step 3.2.3 finally, copy the created .class.violet file (if it exists)
                Main.copy2bootDir(dn + ".class.violet", true);
            }
        }

        // Step 4: foreach vm arrow, generate m corresponding stub file
        Table methodTable = db.getTable("method");
        for (Tuple m : methodTable.getTuples()) {
            if (m.get("ext").equals("vm")) {
                currentFile = libvm + m.get("exe");
                out = setOut(currentFile);
                out.format("#set($MARKER=\"//----\")\n");
                out.format("${MARKER}${OutputFileName}\n");
                out.format("You forgot to write this\n");
                out.format("velocity template %s!\n", currentFile);
                out.close();
            }
        }

        // Step 5: foreach pl arrow, generate corresponding stub file
        for (Tuple m : methodTable.getTuples()) {
            if (m.get("ext").equals("pl")) {
                currentFile = libpl + m.get("filename") + ".pl";
                out = setOut(currentFile);
                out.format("run:-write('please write a proper %s file!').\n", currentFile);
                out.close();
            }
        }

        // Step 6: foreach java arrow, generate corresponding stub file
        for (Tuple m : methodTable.getTuples()) {
            if (m.get("ext").equals("java")) {
                createNecessaryDirectories(m.get("filename"));
            }
        }
    }

    private static void createNecessaryDirectories(String path) {
        String dirs[] = path.split("\\.");
        String dname = srcdir;
        for (int i = 0; i < dirs.length - 1; i++) {
            dname = dname + dirs[i];
            new File(dname).mkdir();
            if (!dname.endsWith("/")) {
                dname = dname + "/";
            }
        }
        dname = dname + dirs[dirs.length-1] + ".java";
        try {
            PrintStream out = new PrintStream(dname);
            out.format("/* please write a proper %s file */\n", dname);
            out.close();
        }
        catch (Exception e) {
            throw new Error("Cant write file " + dname);
        }
    }

    static PrintStream setOut(String filename) {
        try {
            out = new PrintStream(filename);
        } catch (Exception e) {
            throw new Error("can't open " + filename);
        }
        return out;
    }

    public static void makeSubDirectories(String... list) {
        for (String s : list) {
            String dname = "boot/" + s;
            File dir = new File(dname);
            if (!dir.mkdir()) {
                throw new Error("cannot create directory " + dname);
            }
        }
    }

}
