package avalon.tometamodel;

import PrologDB.Table;
import PrologDB.ReadPrologDB;
import PrologDB.Tuple;
import CoreMDELite.Error;
import CoreMDELite.Globals;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.util.Map;
import static avalon.tometamodel.GenDomainClasses.out;
import static avalon.tometamodel.common.packageName;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;

public class GenMainClasses extends common {

    static Table pathTable;
    static Table classTable;
    static Table methodTable;

    public static void marquee() {
        System.err.format("Usage: %s <filename.meta.pl>\n", GenMainClasses.class.getName());
        System.err.format("       directory <filename>/ will be created");
        System.exit(1);
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String... args) {
        // Step 1: let's make sure we open everything properly
        if (args.length != 1) {
            marquee();
        }
        if (init) {
            commonstuff(args);
        }
        initIndent();

        try {
            inputFile = new File(localDbName);
        } catch (Exception e) {
            throw new Error("can't open file " + localDbName);
        }
        try {
            eps = new PrintStream(Globals.errorFile);
        } catch (Exception e) {
            throw new Error("can't open file " + Globals.errorFile);
        }
        db = ReadPrologDB.readDataBase(inputFile, eps);
        eps.close();
        // schema is m map of (tableName,array-list-of-column-names)
        // tables is m map of (tableName, list-of-tuples) where m tuple is m map 
        // (columnName,value) pairs

        // Step 2: load all the tables
        pathTable = db.getTable("path");
        classTable = db.getTable("class");
        methodTable = db.getTable("method");

        // Step 3: generate Main and Debug
        genMain();
        genDebug();
    }

    static void genDebug() {
        PrintWriter out = setOut("Debug.java");
        out.format("package %s;\n\n", packageName);
        out.format("import CoreMDELite.Globals;\n\n");
        out.format("public class Debug extends Main {\n\n");
        out.format("    // same as Main.main, except that Globals.cleanup is set to false -- leave the debris\n");
        out.format("    // meaning that all instances of terminal AND intermediate domains remain\n\n");
        out.format("    public static void main(String... args) {\n");
        out.format("        Globals.cleanup = false;\n");
        out.format("        doit(args);\n");
        out.format("    }\n\n");
        out.format("}\n");
        out.close();
    }

    static void genMain() {
        PrintWriter out = setOut("Main.java");
        out.format("package %s;\n\n", packageName);

        out.format("import CoreMDELite.ConformanceError;\n");
        out.format("import CoreMDELite.Globals;\n");
        out.format("import java.io.File;\n\n");

        out.format("public class Main {\n\n");

        out.format("    static {\n");
        out.format("        Globals.appPath = Globals.getPath(%s.Main.class);\n", packageName);
        out.format("    }\n\n");

        out.format("    public static void setAppPath(boolean printme) {\n");
        out.format("        if (printme) {\n");
        out.format("            System.out.println(\"AppPath : \" + Globals.appPath);\n");
        out.format("        }\n");
        out.format("    }\n\n");

        out.format("    public static void marquee() {\n");
        out.format("        System.err.format(\"\\nUsage: %s <option> <files>\\n\",Main.class.getName());\n", "%s");
        out.format("        System.err.format(\"Format: <file> = <filename>.<domain>.<domainType>\\n\\n\");\n");
        out.format("        System.err.format(\"Option: conform <classname> <filename>\\n\");\n");
        LinkedList<String> stringlist = new LinkedList<>();
        for (Tuple c : classTable.getTuples()) {
            stringlist.add(c.get("name"));
        }
        java.util.Collections.sort(stringlist);
        String list = "";
        for (String s : stringlist) {
            list = list + " " + s;
        }
        out.format("        System.err.format(\"        <classname> in {%s }\\n\");\n", list);
        out.format("        buildlist();\n");
        out.format("        System.err.format(\"        coordinates\\n\");\n");
        out.format("        System.exit(1);\n");
        out.format("    }\n\n");

        out.format("    public static void main(String... args) {\n");
        out.format("        Globals.cleanup = true;\n");
        out.format("        try {\n");
        out.format("            doit(args);\n");
        out.format("        } catch (Error e) {\n");
        out.format("            System.err.format(\"Runtime Error!\\n\\n\");\n");
        out.format("            Utils.PrintFile.main(Globals.errorFile);\n");
        out.format("            return;\n");
        out.format("        } catch (ConformanceError e) {\n");
        out.format("            System.err.println(\"Conformance Error!\\n\");\n");
        out.format("            Utils.PrintFile.main(Globals.conformFile);\n");
        out.format("            return;\n");
        out.format("        } catch (Exception e) {\n");
        out.format("            System.err.println(e.getMessage());\n");
        out.format("        }\n");
        out.format("    }\n\n");

        out.format("    public static void doit(String... args) {\n");
        out.format("        if (args.length == 0) {\n");
        out.format("            marquee();\n");
        out.format("        } else {\n");
        out.format("            switch (args[0]) {\n");
//        out.format("                case \"conform\":\n");
//        out.format("                    conform(args);\n");
//        out.format("                    break;\n");

        stringlist.clear();
        Table pathTable = db.getTable("path");
        for (Tuple c : pathTable.getTuples()) {
            stringlist.add(c.get("name"));
        }
        stringlist.add("coordinates");
        stringlist.add("conform");
        java.util.Collections.sort(stringlist);
        for (String s : stringlist) {
            out.format("                case \"%s\":\n", s);
            out.format("                    %s(args);\n", s);
            out.format("                    break;\n");
        }
        out.format("                default:\n");
        out.format("                    System.out.println(\"Unrecognized option: \" + args[0]);\n");
        out.format("                    System.out.println();\n");
        out.format("                    marquee();\n");
        out.format("            }\n");
        out.format("        }\n");
        out.format("        new File(Globals.conformFile).delete();\n");
        out.format("        new File(Globals.errorFile).delete();\n");
        out.format("    }\n\n");

        // generate 2nd half of main
        initIndent();
        incIndent();
        out.format("%sstatic public void coordinates(String args[]) {\n", indent);
        out.format("%s  System.out.println(\"HomePath = \" + Globals.homePath);\n", indent);
        out.format("%s  System.out.println(\"AppPath  = \" + Globals.appPath);\n", indent);
        out.format("%s}\n\n", indent);

        Collections.sort(pathTable.getTuples(),new sortInterface());
        
        for (Tuple p : pathTable.getTuples()) {
            initVars();
            String pname = p.get("name");
            out.format("%sstatic public void %s(String args[]) {\n", indent, pname);
            incIndent();
            out.format("%sif (args.length != 2) {\n", indent);
            incIndent();
            out.format("%smarquee();\n", indent);
            out.format("%sreturn;\n", indent);
            decIndent();
            out.format("%s}\n", indent);
            out.format("%sString filename = args[1];\n", indent);
            boolean firstTimeThru = true;
            String svname;
            String evname = " null ";  // should never be used

            String path = p.get("path");
            String lastVar = " error ";  // should not be used
            String[] segments = path.split("\\.");
            for (String step : segments) {
                if (step.equals("]")) {
                    decIndent();
                    out.format("%s}\n", indent);
                    evname = " null ";
                    lastVar = " error ";  // this is the end of a loop -- there is no last variable
                    continue;
                }
                if (step.equals("#")) {
                    // increment last variable name
                    if (evname.equals(" null ")) {
                        throw new Error(" illegal use of # in path " + pname);
                    }
                    out.format("%s%s.newVersionNumber();\n", indent, evname);
                    lastVar = evname;
                    continue;
                }
                boolean returnsArray = step.endsWith("[");
                step = step.replace("[", "");
                Tuple method = methodTable.findFirst("name", step);  //findTuple(methodTable, "name", step);
                Tuple resulttype = classTable.findFirst("name", method.get("resulttype").replace("[]", "")); // findTuple(classTable, "name", method.get("resulttype").replace("[]", ""));
                String enddomain = resulttype.get("name");
                Tuple startType = classTable.findFirst("id", method.get("cid")); //findTuple(classTable, "id", method.get("cid"));
                String startdomain = startType.get("name");
                boolean hasAtLeast1Arg = !(method.get("sig").equals(""));
                if (firstTimeThru) {
                    evname = svname = newVar(startdomain);
                    out.format("%s%s %s = new %s(filename);\n", indent, startdomain, evname, startdomain);
                    firstTimeThru = false;
                    lastVar = evname;
                } else {
                    svname = vars.get(startdomain);
                }
                if (returnsArray) {
                    String eva = newVar(enddomain + "[");
                    out.format("%sjava.util.List<%s> %s = %s.%s();\n", indent, enddomain, eva, svname, step);
                    String iterVar = newVar(enddomain);
                    out.format("%sfor (%s %s : %s) {\n", indent, enddomain, iterVar, eva);
                    incIndent();
                    evname = iterVar;
                    lastVar = " error1 "; // there shouldn't be a last variable
                    continue;
                }
                if (hasAtLeast1Arg) {
                    svname = getVar(startdomain);
                    evname = newVar(enddomain);
                    String sig = method.get("sig");
                    String[] params = sig.split(",");
                    String args = "";
                    String comma = "";
                    for (String param : params) {
                        args = args + comma + getVar(param);
                        comma = ",";
                    }
                    out.format("%s%s %s = %s.%s(%s);\n", indent, enddomain, evname, svname, step, args);
                    lastVar = evname;
                } else {
                    svname = getVar(startdomain);
                    evname = newVar(enddomain);
                    out.format("%s%s %s = %s.%s();\n", indent, enddomain, evname, svname, step);
                    lastVar = evname;
                }
            }
            out.format("%sSystem.out.format(\"file %s was produced.\\n\", %s.fullName);\n", indent, "%s", lastVar);
            decIndent();
            out.format("%s}\n\n", indent);
        }

        // Step 5: generate build list
        initIndent();
        incIndent();
        out.format("%sstatic void buildlist() {\n", indent);
        incIndent();
        for (Tuple p : pathTable.getTuples()) {
            String pname = p.get("name");
            String path = p.get("path");
            String[] segments = path.split("\\.");
            String step = segments[0].replace("[", "");

            Tuple meth = methodTable.findFirst("name", step); //findTuple(methodTable, "name", step);
            Tuple classOfMeth = classTable.findFirst("id", meth.get("cid")); // findTuple(classTable, "id", meth.get("cid"));
            String dom = classOfMeth.get("domain");
            String ext = classOfMeth.get("ext");
            out.format("%sSystem.err.format(\"        %-15s <filename>    // of type %s.%s\\n\");\n", indent, pname, dom, ext);
        }
        decIndent();
        out.format("%s}\n\n", indent);

        // Step 6: generate the conform option
        initIndent();
        incIndent();
        initVars();
        out.format("%spublic static void conform(String args[]) {\n", indent);
        incIndent();
        out.format("%sif (args.length != 3) {\n", indent);
        incIndent();
        out.format("%smarquee();\n", indent);
        out.format("%sreturn;\n", indent);
        decIndent();
        out.format("%s}\n", indent);
        out.format("%sString classname = args[1];\n", indent);
        out.format("%sString filename = args[2];\n\n", indent);
        out.format("%sswitch (classname) {\n", indent);

        Collections.sort(classTable.getTuples(),new sortInterface());
        
        for (Tuple c : classTable.getTuples()) {
            String cn = c.get("name");
            String cv, en, ev, ec, dc;
            Map<String, String> m;

            if (c.get("ext").equals("pl")) {
                cv = newVar(cn);
                //dc = className(cn,c.get("ext"));
                incIndent();
                out.format("%scase \"%s\":\n", indent, cn);
                incIndent();
                out.format("%s%s %s = new %s(filename);\n", indent, cn, cv, cn);
                out.format("%s%s.conform();\n", indent, cv);
                out.format("%sbreak;\n", indent);
                decIndent();
                decIndent();
                continue;
            }
            boolean foundit = false;
            
            for (Tuple meth : methodTable.getTuples()) {
                if (meth.get("cid").equals(c.get("id"))) {
                    String resulttype = meth.get("resulttype");
                    Tuple dpl = classTable.findFirst("name", resulttype);       //findTuple(classTable, "name", resulttype);
                    if (dpl.get("ext").equals("pl")) {
                        cv = newVar(cn);
                        dc = c.get("name");
                        incIndent();
                        out.format("%scase \"%s\":\n", indent, cn);
                        incIndent();
                        out.format("%s%s %s = new %s(filename);\n", indent, dc, cv, dc);
                        out.format("%s%s.%s();\n", indent, cv, meth.get("name"));
                        out.format("%sbreak;\n", indent);
                        decIndent();
                        decIndent();
                        foundit = true;
                        break;
                    }
                }
            }
            if (!foundit) {
                // no way to "conform it"
                incIndent();
                out.format("%scase \"%s\":\n", indent, cn);
                incIndent();
                out.format("%sSystem.out.println(\"no conformance rules\");\n", indent, cn);
                out.format("%sbreak;\n", indent);
                decIndent();
                decIndent();
                continue;
            }
        }
        incIndent();
        out.format("%sdefault:\n", indent);
        incIndent();
        out.format("%sSystem.out.println(\"unrecognizable classname \" + classname);\n", indent);
        out.format("%sSystem.out.println();\n", indent);
        out.format("%smarquee();\n", indent);
        decIndent();
        decIndent();
        out.format("%s}\n", indent);
        decIndent();
        out.format("%s}\n", indent);
        out.format("}\n");
        out.flush();
        out.close();
    }

    static PrintWriter setOut(String filename) {
        String RealFileName = "boot/src/" + packageName + "/" + filename;
        try {
            out = new PrintWriter(new BufferedWriter(new FileWriter(RealFileName)));
        } catch (Exception e) {
            throw new Error("can't open " + filename);
        }
        return out;
    }

    static class sortInterface implements Comparator<Tuple> {
        @Override
        public int compare(Tuple t, Tuple t1) {
            return t.get("name").compareTo(t1.get("name"));
        }
    }
}
