package cdexchange;

import CoreMDELite.ConformanceError;
import CoreMDELite.Globals;
import java.io.File;

public class Main {

    static {
        Globals.appPath = Globals.getPath(cdexchange.Main.class);
    }

    public static void setAppPath(boolean printme) {
        if (printme) {
            System.out.println("AppPath : " + Globals.appPath);
        }
    }

    public static void marquee() {
        System.err.format("\nUsage: %s <option> <files>\n",Main.class.getName());
        System.err.format("Format: <file> = <filename>.<domain>.<domainType>\n\n");
        System.err.format("Option: conform <classname> <filename>\n");
        System.err.format("        <classname> in { Dot Kielerdot Nopos Pos Sdb Start Violet VioletDB Yuml YumlDB }\n");
        buildlist();
        System.err.format("        coordinates\n");
        System.exit(1);
    }

    public static void main(String... args) {
        Globals.cleanup = true;
        try {
            doit(args);
        } catch (Error e) {
            System.err.format("Runtime Error!\n\n");
            Utils.PrintFile.main(Globals.errorFile);
            return;
        } catch (ConformanceError e) {
            System.err.println("Conformance Error!\n");
            Utils.PrintFile.main(Globals.conformFile);
            return;
        } catch (Exception e) {
            System.err.println(e.getMessage());
        }
    }

    public static void doit(String... args) {
        if (args.length == 0) {
            marquee();
        } else {
            switch (args[0]) {
                case "conform":
                    conform(args);
                    break;
                case "coordinates":
                    coordinates(args);
                    break;
                case "help":
                    help(args);
                    break;
                case "violet2yuml":
                    violet2yuml(args);
                    break;
                case "yuml2violet":
                    yuml2violet(args);
                    break;
                default:
                    System.out.println("Unrecognized option: " + args[0]);
                    System.out.println();
                    marquee();
            }
        }
        new File(Globals.conformFile).delete();
        new File(Globals.errorFile).delete();
    }

    static public void coordinates(String args[]) {
      System.out.println("HomePath = " + Globals.homePath);
      System.out.println("AppPath  = " + Globals.appPath);
    }

    static public void help(String args[]) {
        if (args.length != 2) {
            marquee();
            return;
        }
        String filename = args[1];
        Start v0 = new Start(filename);
        Start v1 = v0.getStarted();
        System.out.format("file %s was produced.\n", v1.fullName);
    }

    static public void violet2yuml(String args[]) {
        if (args.length != 2) {
            marquee();
            return;
        }
        String filename = args[1];
        Violet v0 = new Violet(filename);
        VioletDB v1 = v0.vclass2violet();
        Sdb v2 = v1.violet2sdb();
        v2.newVersionNumber();
        YumlDB v3 = v2.sdb2yuml();
        Yuml v4 = v3.toYclass();
        System.out.format("file %s was produced.\n", v4.fullName);
    }

    static public void yuml2violet(String args[]) {
        if (args.length != 2) {
            marquee();
            return;
        }
        String filename = args[1];
        Yuml v0 = new Yuml(filename);
        YumlDB v1 = v0.yclass2yuml();
        Sdb v2 = v1.yuml2sdb();
        v2.newVersionNumber();
        Nopos v3 = v2.toNoPos();
        Dot v4 = v3.toDot();
        Kielerdot v5 = v4.Kieler();
        Pos v6 = v5.toPos();
        Sdb v7 = v6.merge(v3);
        VioletDB v8 = v7.sdb2violet();
        Violet v9 = v8.toVclass();
        System.out.format("file %s was produced.\n", v9.fullName);
    }

    static void buildlist() {
        System.err.format("        help            <filename>    // of type start.tmp\n");
        System.err.format("        violet2yuml     <filename>    // of type class.violet\n");
        System.err.format("        yuml2violet     <filename>    // of type yuml.yuml\n");
    }

    public static void conform(String args[]) {
        if (args.length != 3) {
            marquee();
            return;
        }
        String classname = args[1];
        String filename = args[2];

        switch (classname) {
            case "Dot":
                System.out.println("no conformance rules");
                break;
            case "Kielerdot":
                Kielerdot v0 = new Kielerdot(filename);
                v0.toPos();
                break;
            case "Nopos":
                Nopos v1 = new Nopos(filename);
                v1.conform();
                break;
            case "Pos":
                Pos v2 = new Pos(filename);
                v2.conform();
                break;
            case "Sdb":
                Sdb v3 = new Sdb(filename);
                v3.conform();
                break;
            case "Start":
                System.out.println("no conformance rules");
                break;
            case "Violet":
                Violet v4 = new Violet(filename);
                v4.vclass2violet();
                break;
            case "VioletDB":
                VioletDB v5 = new VioletDB(filename);
                v5.conform();
                break;
            case "Yuml":
                Yuml v6 = new Yuml(filename);
                v6.yclass2yuml();
                break;
            case "YumlDB":
                YumlDB v7 = new YumlDB(filename);
                v7.conform();
                break;
            default:
                System.out.println("unrecognizable classname " + classname);
                System.out.println();
                marquee();
        }
    }
}
