package cdexchange.classVioletParser;

import CoreMDELite.ConformanceError;
import CoreMDELite.Error;
import CoreMDELite.Globals;
import java.io.PrintStream;

public class Main {
    static String inputFile;
    static PrintStream errOut;
    static boolean hasErrors;

    /**
     * Main
     */
    public static void main(String... args) {
        
        // Standard marquee test
        if (args.length != 2) {
            System.err.format("Usage: %s <in>.violet <out>.pl\n",Main.class.getName());
            System.exit(1);
        }
        inputFile = args[0];   
        try {
            hasErrors = false;
            errOut = new PrintStream(Globals.conformFile);
        }
        catch (Exception e) {
            throw new Error("can't open " + Globals.conformFile);
        }

        //Import XML data into Java objects
        Importer importer = new Importer(inputFile);
        importer.importNodesAndEdges();
        importer.assignParents();

        //Write our Violet DB file
        DBWriter dbWriter = new DBWriter(args[1], importer.getClassNodes(), importer.getClassEdges(),
                importer.getClassInterfaces(), importer.getVioletData(), importer.getNumNodes());
        dbWriter.generateVioletDB();
        
        // Finally, announce errors
        errOut.close();
        if (hasErrors) {
            throw new ConformanceError("See " + Globals.conformFile);
        }
    }
    
    public static void errout(String s) {
        errOut.println(s);
        hasErrors = true;
    }
    
    public static String makeFirstLowerCase(String s) {
       boolean quotes = false;
       if (s.startsWith("\"")) {
           s = s.replace("\"", "");
           quotes = true;
       }
       String tail = s.substring(1);
       String head = s.substring(0,1).toLowerCase();
       String result = head+tail;
       if (quotes) {
           result =  "\"" +result + "\"";
       }
       return result;
    }
}
