/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package cdexchange.classVioletTParser;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;

/**
 *
 * @author trentkan
 */
public class Main {

    // Data containers

    private static ArrayList<Node> nodes;
    private static ArrayList<Edge> edges;
    private static ArrayList<MiddleLabel> middleLabels;

    //Input output stuff
    private static String inputFileName;
    private static String outFileName;
    private static FileWriter fw;

    public static void main(String... args) {
        if (args.length != 2) {
            System.err.format("Usage: %s <in>.violet.html <out>.pl\n", Main.class.getName());
            System.exit(1);
        }
        // Get file names from args
        inputFileName = args[0];
        outFileName = args[1];
        // Create parser object to parse input file
        Parser p = new Parser(inputFileName);
        nodes = p.getNodeList();
        edges = p.getEdgeList();
        middleLabels = p.getMidLabelList();

        try {
            fw = new FileWriter(new File(outFileName));

        } catch (IOException e) {
            System.out.println(e);
        }
        // Print at the beginning
        writeLine(":- style_check(-discontiguous).");
//        writeLine("table(violetTClass,[id, \"name\",\"fields\",\"methods\",x,y]).");
//        writeLine("table(violetTInterface,[id,\"name\",\"methods\",x,y]).");
//        writeLine("table(violetTAssociation,[id,\"role1\",\"arrow1\",type1,\"role2\",\"arrow2\",type2,\"lineStyle\",cid1,cid2]).");
//        writeLine("table(violetTMiddleLabels,[id,cid1,cid2,\"label\"]).");
        writeLine("");
        // Parse the XML/HTML file file
        p.parse();
        // Print the tuples based on the data parsed
        printTuples(p);
        try {
            fw.flush();
            fw.close();
        } catch (IOException e) {
            System.out.println(e);
        }
    }

    private static void printTuples(Parser p) {
        printClassNodes(p);
        printInterfaceNodes(p);
        printEdges(p);
        printMiddleLabels(p);
    }

    // Prints the classes
    private static void printClassNodes(Parser p) {
        writeLine("table(violetClass,[id,\"name\",\"fields\",\"methods\",x,y]).");
        if (p.numClasses == 0) {
            writeLine(":- dynamic violetClass/6.");
        } else {
            for (Node n : nodes) {
                if (n.isClass()) {
                    writeLine(n.toString());
                }
            }
        }
        writeLine("");
    }

    // Prints the interfaces
    private static void printInterfaceNodes(Parser p) {
        writeLine("table(violetInterface,[id,\"name\",\"methods\",x,y]).");
        if (p.numInterfaces == 0) {
            writeLine(":- dynamic violetInterface/5.");
        } else {
            for (Node n : nodes) {
                if (!n.isClass()) {
                    writeLine(n.toString());
                }
            }
        }
        writeLine("");
    }

    // Prints the edges
    private static void printEdges(Parser p) {
        writeLine("table(violetAssociation,[id,\"role1\",\"arrow1\",type1,\"role2\",\"arrow2\",type2,\"lineStyle\",cid1,cid2]).");
        if (p.numExtendEdges == 0 && p.numImplementEdges == 0
                && p.numDependencyEdges == 0 && p.numAssociationEdges == 0
                && p.numAggregationEdges == 0 && p.numCompositionEdges == 0) {
            writeLine(":- dynamic violetAssociation/10.");
        } else {
            for (Edge e : edges) {
                writeLine(e.toString());
            }
        }
        writeLine("");
    }

    // Prints middle labels
    private static void printMiddleLabels(Parser p) {
        writeLine("table(violetMiddleLabels,[id,cid1,cid2,\"label\"]).");
        if (middleLabels.size() == 0) {
            writeLine(":- dynamic violetMiddleLabels/4.");
        } else {
            for (MiddleLabel m : middleLabels) {
                writeLine(m.toString());
            }
        }
    }

    private static String quote(String original) {
        return "\'" + original + "\'";
    }

    private static void writeLine(String toWrite) {
        try {
            fw.write(toWrite);
            fw.write("\n");
        } catch (IOException e) {
            System.out.println("Error writing file: " + e);
        }
    }
}
