/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package cdexchange.classVioletTParser;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;

import cdexchange.classVioletTParser.Globals.EdgeType;


/**
 *
 * @author trentkan
 */
public class Parser {
    
    public static int numClasses = 0;
    public static int numInterfaces = 0;
    public static int numExtendEdges = 0;
    public static int numImplementEdges = 0;
    public static int numDependencyEdges = 0;
    public static int numAggregationEdges = 0;
    public static int numAssociationEdges = 0;
    public static int numCompositionEdges = 0;
    
    private static int cid = 0;
    private static int iid = 0;
    private static int aid = 0;
    private static int mid = 0;
    
    private static HashMap<String, String> violet2db_ids;
    private static ArrayList<Node> nodes;
    private static ArrayList<Edge> edges;
    private static ArrayList<MiddleLabel> middleLabels;
    
    private final static int SPACES_TO_ID = 5;
    private final static String CLASS_ID = "classnode";
    private final static String INTERFACE_ID = "interfacenode";
    private final static String ASSOCIATION_ID = "association";
    private final static String MIDDLELABEL_ID = "middlelabel";

    private static String inputFileName;
    
    public Parser(String inputFile) 
    {
        initCounters();
        violet2db_ids = new HashMap<>();
        nodes = new ArrayList<>();
        edges = new ArrayList<>();
        middleLabels = new ArrayList<>();
        inputFileName = inputFile;
    }
    
    // Resets counter variables
    private void initCounters()
    {
        numClasses = 0;
        numInterfaces = 0;
        numExtendEdges = 0;
        numImplementEdges = 0;
        numDependencyEdges = 0;
        numAggregationEdges = 0;
        numAssociationEdges = 0;
        numCompositionEdges = 0;

        cid = 0;
        iid = 0;
        aid = 0;
        mid = 0;
    }
    
    public void parse()
    {
        countFile();
        parseFile();
    }
    
    // Reads the file and counts the number of 
    // classes, interfaces, and edges
    public static void countFile()
    {
        try
        {
            FileInputStream f = new FileInputStream(inputFileName);
            boolean cont = true;
            while(cont)
            {
                skipTo(f, "<");
                String tag = scanTag(f);
                switch (tag)
                {
                    case "/ClassNode":
                        numClasses++;
                        break;
                    case "/InterfaceNode":
                        numInterfaces++;
                        break;
                    case "/InheritanceEdge":
                        numExtendEdges++;
                        break;
                    case "/InterfaceInheritanceEdge":
                        numImplementEdges++;
                        break;
                    case "/DependencyEdge":
                        numDependencyEdges++;
                        break;
                    case "/AggregationEdge":
                        numAggregationEdges++;
                        break;
                    case "/AssociationEdge":
                        numAssociationEdges++;
                        break;
                    case "/CompositionEdge":
                        numCompositionEdges++;
                        break;           
                    case "/HTML":
                        cont = false;
                        break;
                }
            }
            f.close(); 
        }
        catch (Exception e)
        {
            System.out.println(e);
        }
        
    }
    
    // Handles the parsing of classes, interfaes
    // and edges
    public static void parseFile()
    {
        if(numClasses != 0)
        {
            processShape("<ClassNode");
        }
        if(numInterfaces != 0)
        {
            processShape("<InterfaceNode");
        }
        if(numExtendEdges != 0)
        {
            processShape("<InheritanceEdge");
        }
        if(numImplementEdges != 0)
        {
            processShape("<InterfaceInheritanceEdge");
        }
        if(numDependencyEdges != 0)
        {
            processShape("<DependencyEdge");
        }
        if(numAssociationEdges != 0)
        {
            processShape("<AssociationEdge");
        }
        if(numAggregationEdges != 0)
        {
            processShape("<AggregationEdge");
        }
        if(numCompositionEdges != 0)
        {
            processShape("<CompositionEdge");
        }
    }
    
    public static void processShape(String type)
    {
        try
        {
            FileInputStream f = new FileInputStream(inputFileName);
            skipToTag(f, "BODY");
            switch(type)
            {
                case "<ClassNode":
                    skipToNode(f, numClasses, type, true);
                    break;
                case "<InterfaceNode":
                    skipToNode(f, numInterfaces, type, false);
                    break;
                case "<InheritanceEdge":
                    skipToEdge(f, numExtendEdges, type, EdgeType.EXTENDS);
                    break;
                case "<InterfaceInheritanceEdge":
                    skipToEdge(f, numImplementEdges, type, EdgeType.IMPLEMENTS);
                    break;
                case "<DependencyEdge":
                    skipToEdge(f, numDependencyEdges, type, EdgeType.DEPENDS);
                    break;
                case "<AssociationEdge":
                    skipToEdge(f, numAssociationEdges, type, EdgeType.ASSOCIATION);
                    break;
                case "<AggregationEdge":
                    skipToEdge(f, numAggregationEdges, type, EdgeType.AGGREGATION);
                    break;
                case "<CompositionEdge":
                    skipToEdge(f, numCompositionEdges, type, EdgeType.COMPOSITION);
                    break;
            }
            f.close();
        }
        catch (Exception e)
        {
            System.out.println(e);
        }
    }
    
    // Finds a node tag (starts with <ClassNode or <InterfaceNode
    // and then processes it
    private static void skipToNode(FileInputStream f, int limit, String type, boolean isClass) throws IOException
    {
        for(int i = 0; i < limit; ++i)
        {
            skipTo(f, type);                                              
            processNode(f, isClass);
            String endTag = type.charAt(0) + "/" + type.substring(1);
            skipTo(f, endTag); 
        }
    }
    
    // Finds an edge tag (starts with <AssociationEdge, <DependencyEdge, etc.
    // and then processes it
    private static void skipToEdge(FileInputStream f, int limit, String type, EdgeType et) throws IOException
    {
        for(int i = 0; i < limit; ++i)
        {
            skipTo(f, type);                                              
            processEdge(f, et);
            String endTag = type.charAt(0) + "/" + type.substring(1);
            skipTo(f, endTag); 
        }
    }
    
    // Skips to a tag named comp
    private static void skipToTag(FileInputStream f, String comp) throws IOException
    {
        int c = f.read();
        while(c != -1)
        {
            if((char)c == '<')
            {
                String result = scanTag(f);
                if(result.equals(comp))
                {
                    break;
                }
                else
                {
                    c = f.read();
                }
            }
            else
            {
                c = f.read();
            }
        }
    }
    
    // Skips to string target
    private static void skipTo(FileInputStream f, String target) throws IOException
    {
        int c = f.read();
        while (c != -1)
        {
            boolean found = true;
            for(int i = 0; i < target.length(); ++i)
            {
                if((char)c != target.charAt(i))
                {
                    found = false;
                    break;
                }
                else if(i != (target.length() - 1))
                {
                    c = f.read(); 
                }
            }
            if(found)
            {
                break;
            }
            else
            {
                c = f.read();
            }
        }
    }
    
    // Reads in characters of an HTML tag until it reaches >
    public static String scanTag(FileInputStream f) throws IOException
    {
        StringBuilder sb = new StringBuilder();
        char c = (char) f.read();
        while(c != '>')
        {
            sb.append(c);
            c = (char) f.read();
        }
        return sb.toString(); 
    }
    
    // Similar to scanTag except it stops at stopValue instead of >
    public static String gatherInfo(FileInputStream f, char stopValue) throws IOException
    {
        StringBuilder sb = new StringBuilder();
        int c = f.read();
        while((char)c != stopValue)
        {
            sb.append((char)c);
            c = f.read();
        }
        return sb.toString();
    }
    
    // Gathers necessary information for a node, packages the information into
    // an array, then adds the node to an node list
    public static void processNode(FileInputStream f, boolean isClass) throws IOException
    {
        for(int i = 0; i < SPACES_TO_ID; ++i)
        {
            f.read();
        }
        String id = gatherInfo(f, '\"');
        
        skipTo(f, "x=");
        f.read(); // Advance past "
        String xValue = gatherInfo(f, '\"');

        skipTo(f, "y=");
        f.read(); // Advance past "
        String yValue = gatherInfo(f, '\"');

        skipToTag(f, "text");
        String name = gatherInfo(f, '<');

        skipToTag(f, "text");
        String attributes = "";
        if(isClass)
        {
            attributes = gatherInfo(f, '<');
            if(!attributes.equals(""))
            {
                attributes = attributes.replace('\n', ';');
            }
            skipToTag(f, "text");
        }
        String methods = gatherInfo(f, '<');
        methods = methods.replace('\n', ';');
        String db;
        if(isClass)
        {
            db = CLASS_ID + cid;
            cid++;
            violet2db_ids.put(id, db);
        }
        else
        {
            db = INTERFACE_ID + iid;
            iid++;
            violet2db_ids.put(id, db);
        }
        
        String[] params = {db, name, attributes, methods, xValue, yValue};
        nodes.add(new Node(params, isClass));
    }
    
    // Gathers necessary information for an edge, packages the information into
    // an array, then adds the edge to an edge list
    public static void processEdge(FileInputStream f, EdgeType et) throws IOException
    {
        boolean interfaceStart = false;
        boolean interfaceEnd = false;

        String id = ASSOCIATION_ID + aid;
        aid++;
        
        skipTo(f, "class=");
        f.read(); // Advance past "
        String classOrInterface = gatherInfo(f, '\"');
        interfaceStart = classOrInterface.equals("InterfaceNode");
        
        skipTo(f, "reference=");
        f.read(); // Advance past "
        String startID = gatherInfo(f, '\"');
        startID = violet2db_ids.get(startID);
        
        skipTo(f, "class=");
        f.read(); // Advance past "
        classOrInterface = gatherInfo(f, '\"');
        interfaceEnd = classOrInterface.equals("InterfaceNode");
       
        skipTo(f, "reference=");
        f.read(); // Advance past "
        String endID = gatherInfo(f, '\"');
        endID = violet2db_ids.get(endID);

        skipToTag(f, "startLabel");
        String startLabel = gatherInfo(f, '<');
        
        skipToTag(f, "middleLabel");
        String middleLabel = gatherInfo(f, '<');
        // Make middle label
        if(!middleLabel.equals(""))
        {
            String[] middleLabelParams = {MIDDLELABEL_ID + mid, startID, endID, middleLabel};
            mid++;
            middleLabels.add(new MiddleLabel(middleLabelParams));
        }

        skipToTag(f, "endLabel");
        String endLabel = gatherInfo(f, '<');
        
        String[] params = {id, startID, endID, startLabel, middleLabel, endLabel};
        boolean[] iface = {interfaceStart, interfaceEnd};
        edges.add(new Edge(params, et, iface));
    }

    // Returns node list 
    public ArrayList<Node> getNodeList()
    {
        return nodes;
    }
    
    // Returns edge list
    public ArrayList<Edge> getEdgeList()
    {
        return edges;
    }

    // Returns middle label list
    public ArrayList<MiddleLabel> getMidLabelList()
    {
        return middleLabels;
    }
}
