/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package CoreMDELite;

import java.io.*;
import java.util.Formatter;
import org.apache.commons.io.FileUtils;

public abstract class MDELiteObject {

    // here is the non-static definition of an MDELiteObject
    public String fileName;
    public String partialName;
    public String fullName;
    public File file;

    public MDELiteObject(String filename) {
        init(filename);
    }

    private void init(String filename) {
        this.fileName = filename;
        this.partialName = filename + partialFileType();
        this.fullName = filename + fileType();
        file = new File(fullName);
    }

    protected MDELiteObject() {
    } // shouldn't be called

    public abstract String domainName();

    public abstract String domainType();

    public String fileType() {
        return partialFileType() + "." + domainType().toLowerCase();
    }

    public String partialFileType() {
        return "." + domainName().toLowerCase();
    }

    // delete the MDELiteObject
    public void delete() {
        file.delete();
    }

    public void newVersionNumber() {
        String newFileName;

        if (fileName.matches(".*\\d\\d\\d$")) {
            int l = fileName.length();
            //String left = fileName.substring(0,l-3);
            String right = fileName.substring(l - 3);
            int i = Integer.parseInt(right) + 1;
            StringBuilder sb = new StringBuilder();
            Formatter f = new Formatter(sb);
            f.format("_%03d", i);
            newFileName = fileName.substring(0, l - 4) + f;
        } else {
            newFileName = fileName + "_000";
        }

        File src, dst;
        try {
            src = new File(fullName);
            dst = new File(newFileName + fileType());
            if (dst.exists()) {
                dst.delete();
            }
            FileUtils.copyFile(src, dst);
        } catch (Exception e) {
            throw new Error(e.getMessage());
        }
        init(newFileName);
    }

    // spawn -- this spawns a process but doesn't wait for it to finish
    public static Process spawnCommandLine(String... cmdarray) throws Exception {
        return new ProcessBuilder(cmdarray).start();
    }

    // original design of execute
    public static void executeCommandLine(String... cmdarray) throws Exception {
        String line;

        // Step 1: execute the command array, get handles on standard error, standard out, and wait until process is finished.
        //         er is a reader to the process error output
        //         in is a reader to the process standard output
//        Process p = new ProcessBuilder(cmdarray).start();
        Process p = spawnCommandLine(cmdarray);
        BufferedReader er = new BufferedReader(new InputStreamReader(p.getErrorStream()));
        BufferedReader in = new BufferedReader(new InputStreamReader(p.getInputStream()));
        p.waitFor();

        // Step 2: relay all process standard output to sdtout
        while ((line = in.readLine()) != null) {
            System.out.println(line);
        }
        in.close();

        // Step 3: drain process standard error into Globals.errorFile, provided that there is something to output
        boolean error = false;
        if ((line = er.readLine()) != null) {
            PrintStream ps = new PrintStream(Globals.errorFile);
            do {
                ps.format("%s\n", line);
            } while ((line = er.readLine()) != null);
            ps.flush();
            ps.close();
            error = true;
        }
        er.close();

        // Step 4: destroy the process
        p.destroy();

        // Step 5: if there was error output, throw an exception.
        if (error) {
            throw new Error("Consult " + Globals.errorFile);
        }
    }

    public static void done(Exception e) {
        System.err.println(e.getMessage());
        System.exit(1);
    }

    // get a print stream for the current MDELiteObject
    public PrintStream getPrintStreamForCurrentMDELiteObject() {
        PrintStream result = null;
        try {
            result = new PrintStream(file);
        } catch (Exception e) {
            throw new Error(e.getMessage());
        }
        return result;
    }

    // MDELite's contribution to garbage collection
    // instances of all category-interior MDELite domains are flagged
    // as delete-on-exit, unless Globals.cleanup (set by Main or Debug) is set
    public void temporary() {
        if (Globals.cleanup) {
            file.deleteOnExit();
        }
    }
}
