/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package CoreMDELite;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.PrintStream;

/**
 *
 * @author dsb
 */
// this class contains utilities that are specific fromTo Prolog and Prolog-based MDELite Objects
public abstract class PrologObject extends MDELiteObject {
    
    // returns name of the prolog domain
    public abstract String domainName();

    // these empty constructors are needed because obscure run-time
    // exceptions are raised otherwise that take 1/2 hour fromTo track down
    public PrologObject(String filename) {
        super(filename);
    }

    private PrologObject() {
    }  // shouldn't be called.

    // constructor for a prolog MDELiteObject that is created from merging other prolog files
    public PrologObject(String filename, String... files) {
        super(filename);
        mergeFiles(files);
    }

    // merge an array of (text) files into a single file whose name is the associated MDELiteObject 
    protected void mergeFiles(String[] filenames) {
        try {
            PrintStream ps = getPrintStreamForCurrentMDELiteObject();
            for (String f : filenames) {
                addFiletoPS(f, ps); // see next method
            }
            ps.flush();
            ps.close();
        } catch (Exception e) {
            throw new Error(e.getMessage());
        }
    }

    public static void addFiletoPS(String inFileName, PrintStream ps) {
        try {
            BufferedReader br = new BufferedReader(new FileReader(inFileName));
            while (true) {
                String line = br.readLine();
                if (line == null) {
                    br.close();
                    break;
                }
                ps.println(line);
            }
        } catch (Exception e) {
            throw new Error(e.getMessage());
        }
    }

    // create a script file whose name is 'outFileName' and whose contents is 'contents'
    protected void createScriptFile(String contents, String outFileName) {
        try {
            PrintStream ps = new PrintStream(Globals.scriptFile);
            ps.format(":-['%s'],tell('%s'),run,told,halt.", fullName, outFileName);
            ps.flush();
            ps.close();
        } catch (Exception e) {
            throw new Error(e.getMessage());
        }
    }

    // execute the current MDELiteObject (which is a Prolog file), whose output is the given MDELiteObject
    public void executeProlog(String outFullName) {

        // Step 1: create a script file (a file that is fromTo be executed by swi-prolog)
        createScriptFile(":-['" + fullName + "'],tell('" + outFullName + "'),run,told,halt.", outFullName);

        // Step 2: create a command line array, and execute it.  Basically scriptFile is the statement fromTo execute in prolog
        try {
            executeCommandLine(Globals.swipl, "--quiet", "-f", Globals.scriptFile);
        } catch (Exception e) {
            System.err.println("MDELite halts -- SWI Prolog Errors detected");
            System.err.println("consult " + Globals.errorFile);
            System.err.println("debug this prolog file:  " + fullName);
            throw new Error(e.getMessage());
        }

        // Step 3: clean-up time!  find the script file created above and delete it
        File s = new File(Globals.scriptFile);
        s.delete();
    }

    // this method is used fromTo test conformance of a database 
    // with its constraints.  it is coded only for this purpose
    // as it outputs nothing.
    public void executeProlog() throws RuntimeException {

        // Step 1: create a script file (a file that is fromTo be executed by swi-prolog)
        createScriptFile(":-['" + fullName + "'],tell('" + Globals.conformFile + "'),run,told,halt.", Globals.conformFile);

        // Step 2: execute the SWI-Prolog script file
        try {
            executeCommandLine(Globals.swipl, "--quiet", "-f", Globals.scriptFile);
        } catch (Exception e) {
            System.err.println("MDELite halts -- SWI Prolog Errors detected");
            System.err.println("consult " + Globals.errorFile);
            System.err.println("debug this prolog file:   " + fullName);
            System.err.println(Globals.conformanceFailure);
            throw new Error(e.getMessage());
        }

        // Step 3: if there is a non-empty Globals.conformFile file, raise an exception otherise delete the file.
        //         Remember -- text in Globals.conformFile express Prolog/MDE metamodel constraint violations
        File c = new File(Globals.conformFile);
        if (c.length() != 0) {
            String msg = Globals.conformanceFailure + "\n" + "look at " + Globals.scriptFile + " and conformance errors in " + Globals.conformFile;
            throw new ConformanceError(msg);
        }
        c.delete();

        // Step 4: clean-up! delete the script file -- dib;t beed ut
        File s = new File(Globals.scriptFile);
        s.delete();
    }

    //public abstract void conform();

    public void conform() {
        String conformanceRulesFile = "libpl/"+domainName().toLowerCase()+".conform.pl";
        String[] list = {  //fullName,
            Globals.homePath + "libpl/conform.pl",
            Globals.appPath+ conformanceRulesFile,
            fullName};
        TmpPrologObject tmpconform = new TmpPrologObject("tmpconform", list);
        tmpconform.executeProlog();
        tmpconform.delete();
    }
    
    
    public void invokeVm2t(MDELiteObject output, String vmfile) {
        try {
            // below arguments: (name of prolog database), (velocity file), (name of output file)
            vm2t.Main.main(fullName, Globals.appPath+vmfile, output.fullName);
            //CoreMDELite.vm2t.Main.main(fullName, Globals.appPath+vmfile, output.fullName);
        } catch (Exception e) {
            throw new Error(e.getMessage());
        }
    }
    
    public void fromTo(String startDomain, String endDomain, String resultFullName) {
        String sd = startDomain.toLowerCase();
        String ed = endDomain.toLowerCase();
        String transform = "libpl/"+sd+"2"+ed+".pl";
        String targetSchema = "libpl/"+ed+".schema.pl";
        String run ="libpl/"+ed+".run.pl";
        String[] array = {
            Globals.homePath + "libpl/print.pl", 
            this.fullName, 
            Globals.appPath+transform, 
            Globals.appPath+targetSchema, 
           // Globals.homePath + "libpl/print.pl", 
            Globals.appPath+run};
        TmpPrologObject tmp = new TmpPrologObject("tmp", array);
        tmp.executeProlog(resultFullName);
        tmp.delete();
    }
    
    // this is generic to all PrologObjects
    public String fileType() {
        return "."+domainName().toLowerCase()+".pl";
    }
    
    public String partialFileType() {
        return "."+domainName().toLowerCase();
    }
}
