/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package PrologDB;

import CoreMDELite.Error;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

/**
 *
 * @author trentkan
 */
public class ReadPrologDB {
// factory methods for DB, DBSchema

    private static String trimmed(String s) {
        s = s.trim();
        if (s.startsWith("'")) {
            s = s.substring(1, s.length() - 1);
        }
        return s;
    }

    public static DB readDataBase(File infile, PrintStream eps) {
        DB db;
        LineNumberReader br = null;
        boolean hasErrors = false;
//        List<Column> columns;

        // read the file in the first pass: get its schema
        DBSchema dbs = readSchema(infile, eps);
        db = new DB(dbs);

        try {
            // Step 1: Read infile by line
            br = new LineNumberReader(new InputStreamReader(new FileInputStream(infile)));
            String line;
            while ((line = br.readLine()) != null) {
                if (line.startsWith("tuple(") || line.startsWith("dbase(") || line.contains(":-")
                        || line.startsWith("table(") || line.startsWith("subtable(")) {
                    continue;
                }
                // Step 3.1: we found a tuple line -- see if it passes minimal
                //           structure testing
                if (line.matches("\\s*\\w+\\s*(.*)\\.")) {
                    // Step 3.1.1 : get the table name (in tableName)
                    //              the string inbetween '('...')' in strippedLine
                    String tableName = line.split("\\(")[0].trim();
                    String strippedLine = line.substring(line.indexOf("(") + 1, line.lastIndexOf(")"));
                    if (strippedLine.equals("")) {
                        continue;
                    }

                    // Step 3.1.2: if we don't recognize the name of the table, ignore the line
                    try {
                        dbs.findTableSchema(tableName);
                    } catch (Error e) {
                        continue;
                    }

                    // Step 3.1.3: now produce a new string that allows us to split all of its
                    //             pieces by "@@@".
                    String newline = "";
                    boolean inString = false;
                    for (int i = 0; i < strippedLine.length(); i++) {
                        char current = strippedLine.charAt(i);
                        if (current == '\'') {
                            inString = !inString;
                        }
                        if (!inString && current == ',') {
                            newline = newline + "@@@";
                        } else {
                            newline = newline + current;
                        }
                    }

                    String[] columnValues = newline.split("@@@");

                    // Step 3.1.4: get the current set of tuples for the given table in variable tuples 
                    // Get list of tuples (should be empty at this point)
                    Table tbl = db.getTable(tableName);
                    // Create a tuple, really just a HashMap of columnName to string values
                    TableSchema tableDef = db.getTableSchema(tableName);
                    Tuple newTuple = new Tuple(tableDef);
                    // Get a list of columns
                    for (int i = 0; i < tableDef.size(); i++) {
                        // Get the value delimited by "@@@"
                        String value = trimmed(columnValues[i]);
                        // Get the column name 
                        Column target = tableDef.getIndex(i);
                        String columnName = target.getName().replace("\"", "");
                        // Add the column name and value to a tuple struct
                        newTuple.addColumnValue(columnName, value);
                    }
                    // Add the tuple struct to the list of tuples in the table specified by
                    // tableName + "S"
                    tbl.addTuple(newTuple);
                }
            }
            br.close();
        } catch (IOException e) {
            e.printStackTrace();
            hasErrors = true;
        }
        if (hasErrors) {
            throw new Error("errors in parsing prolog table " + infile.getName());
        }
        return db;
    }

    public static DBSchema readSchema(File schemafile, PrintStream eps) {
        DBSchema dbs = null;
        boolean oneDBaseSeen = false;
        List<String> tableNames = new ArrayList<>();

        boolean hasErrors = false;
        try {
            LineNumberReader br = new LineNumberReader(new InputStreamReader(new FileInputStream(schemafile)));
            String line;
            while ((line = br.readLine()) != null) {
                line = line.trim();

                if (line.startsWith("dbase(")) {
                    if (oneDBaseSeen) {
                        throw new Error("more than one dbase() declaration in file " + schemafile.getName());
                    }
                    oneDBaseSeen = true;

                    StringTokenizer st = new StringTokenizer(line, " \t(),][.");
                    try {
                        st.nextToken(); // ignore "dbase"
                        tableNames = new ArrayList<>();
                        while (st.hasMoreTokens()) {
                            // Get column name
                            String name = st.nextToken().trim();
                            tableNames.add(name);
                        }
                        // Remove first string -- this is the name of the database
                        String schemaName = tableNames.remove(0);
                        // later, make sure that all tables in tableNames are represented
                        dbs = new DBSchema(schemaName);
                    } catch (Exception e) {
                        eps.format("unable to parse table on line %d in %s\n",
                                br.getLineNumber(), schemafile.getName());
                        hasErrors = true;
                    }
                }

                // If it's a table line, then add to schema and create new table struct
                if (line.startsWith("table(")) { // parse table facts
                    if (dbs == null) {
                        throw new Error("table( defined before dbase(");
                    }

                    StringTokenizer st = new StringTokenizer(line, " \t(),][.");
                    try {
                        st.nextToken(); // ignore "table"
                        List<Column> columns = new ArrayList<>();
                        while (st.hasMoreTokens()) {
                            // Get column name
                            String name = st.nextToken().trim();
                            boolean isQuoted = name.matches("\".*\"");
                            Column newColumn = new Column(name.replace("\"", ""), isQuoted);
                            columns.add(newColumn);
                        }
                        // Remove first column and get its' name
                        String schemaName = columns.remove(0).getName();
                        // Add schema (first column is schema name, everything else is a column
                        // Add table (first column is table name with empty list of tuples)
                        TableSchema ts = new TableSchema(schemaName, columns);
                        dbs.addTableSchema(ts);
                    } catch (Exception e) {
                        eps.format("unable to parse table on line %d in %s\n",
                                br.getLineNumber(), schemafile.getName());
                        hasErrors = true;
                    }
                }

                if (line.startsWith("subtable(")) { // parse inheritance facts
                    StringTokenizer st = new StringTokenizer(line, " \t(),][.");
                    try {
                        st.nextToken(); // ignore "subtable"
                        ArrayList<TableSchema> names = new ArrayList<>();
                        while (st.hasMoreTokens()) {
                            // Get subtable name
                            String name = st.nextToken().trim();
                            TableSchema ts = dbs.findTableSchema(name);
                            names.add(ts);
                        }
                        // Remove first name == it is the name of the supertable
                        TableSchema supr = names.remove(0);
                        SubTableSchema sts = dbs.findSubTableSchema(supr.getName());
                        if (sts == null) {
                            sts = new SubTableSchema(supr, names);
                            dbs.addSubTableSchema(sts);
                        } else {
                            sts.addSubTableSchemas(names);
                        }
                    } catch (Exception e) {
                        eps.format("unable to parse table on line %d in %s\n",
                                br.getLineNumber(), schemafile.getName());
                        hasErrors = true;
                    }
                }

                // ignore everything else
            }
            br.close();

            // we have parsed a database schema file
            // Let's make sure that the tables declared match what is in the dbase( declaration
            if (dbs == null) {
                throw new Error("dbase() not defined in file " + schemafile.getName());
            }
            if (tableNames.size() != dbs.size()) {
                throw new Error("number of tables defined in dbase() doesn't match # defined in file " + schemafile.getName());
            }
            for (String s : tableNames) {
                dbs.findTableSchema(s);  // throw error if not found
            }
        } catch (IOException e) {
            e.printStackTrace();
            hasErrors = true;
        }
        if (hasErrors) {
            throw new Error("errors in parsing prolog table " + schemafile.getName());
        }
        return dbs;
    }
}
