/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package PrologDB;

import CoreMDELite.Error;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author trentkan
 */
public class TableSchema {

    private String name;
    private List<Column> columns;

    public TableSchema(String name) {
        this.name = makeFirstLowerCase(name);
        columns = new ArrayList<>();
    }

    public TableSchema(String name, List<Column> cols) {
        this.name = name;
        columns = cols;
    }

    public String getName() {
        return name;
    }

    public Column getIndex(int i) {
        return columns.get(i);
    }

    public List<Column> getColumns() {
        return columns;
    }

    public int size() {
        return columns.size();
    }

    public void addColumn(Column c) {
        columns.add(c); // append
    }
    
    public void addColumns(List<Column> c) {
        columns.addAll(0,c);
    }

    public Column findColumn(String name) {
        for (Column c : columns) {
            if (c.getName().equals(name)) {
                return c;
            }
        }
        throw new Error("can't find column " + name + " in table " + this.name);
    }

    public void print(PrintStream out) {
        String q;
        String tb = "table(" + name + ",[";
        String comma = "";
        for (Column c : columns) {
            q = c.isQuoted() ? "\"" : "";
            tb = tb + comma + q + c.getName() + q;
            comma = ",";
        }
        tb = tb + "]).\n";
        out.format(tb);
    }
    
    public void writeTupleDef(PrintStream out) {
        // Step 1: create a list of prolog variable, the # of which equals the # of attributes in the relation
        String accum = varList(0);

        // Step 2: output the tuple definition
        out.format("tuple(%s,L):-%s(%s),L=[%s].\n", name, name, accum, accum);
    }
    
    public void printInheritanceRule(PrintStream out) {
        String pList = varList(0);
        out.format("%sALL(%s):-%s(%s).\n", getName(), pList, getName(), pList);
    }
    
    // create a list of prolog variables, the # of which equals the # of attributes in the relation
    public String varList(int fill) {
        int len = columns.size();
        String accum = "";
        String comma = "";
        char letter = 'A';
        for (int i = 1; i <= (len-fill); i++) {
            accum = accum + comma + letter + i;
            comma = ",";
        }
        for (int j = 1; j <= fill; j++) {
            accum = accum + ",_";
        }
        return accum;
    }
    
    private static String makeFirstLowerCase(String s) {
        String tail = s.substring(1);
        String head = s.substring(0, 1).toLowerCase();
        return head + tail;
    }

    public void equals(TableSchema otherSchema) {
        if (otherSchema == null) {
            throw new Error("otherschema is null");
        }
        if (columns.size() != otherSchema.columns.size()) {
            throw new Error("table schemas (" + name + "," + otherSchema.getName() + ") do not have same # of columns");
        }
        if (!name.equals(otherSchema.getName())) {
            throw new Error("table schemas (" + name + "," + otherSchema.getName() + ") do not have same names");
        }
        for (Column c : columns) {
            c.equals(otherSchema.findColumn(c.getName())); // throw error if not equal
        }
    }
}
