/* file meta.conform.pl */

/* CONSTRAINT: all class names are unique */
cname(I,N):-class(I,N,_,_,_,_,_).
duplicateClassNames(I1,I2,N):-cname(I1,N),cname(I2,N),@I1 @< @I2.

noDuplicateClassNames:-forall(duplicateClassNames(I1,I2,N),isError('two classes have the same name ',[I1,I2,N])).


/* CONSTRAINT: all methods must have unique names -- tools don't work if methods have only unique names within classes*/
mname(I,N):-method(I,_,N,_,_,_,_).
duplicateMethodNames(I1,I2,MN):-mname(I1,MN),mname(I2,MN),@I1 @< @I2.

noDuplicateMethodNames:-forall( duplicateMethodNames(I1,I2,MN), isError('two methods in your class diagram the same name ',[I1,I2,MN])).

/* CONSTRAINT: all methods belong to existing classes (although it is possible that a class has no methods) */

notExist(N):-method(_,Cid,N,_,_,_,_),not(class(Cid,_,_,_,_,_,_)).

methodBelongsToNonExistantClass :- forall( notExist(N),isError('method does not belong to an existing class ',N)).

/* CONSTRAINT: all method extensions are either java,pl,PL,or VM */
ext(java).
ext(pl).
ext(vm).

notLegalExt(I,MN,E):-method( I,_,MN,_,_,_,E),not(ext(E)).

illegalExt:-forall(notLegalExt(I,MN,E), isError('method has illegal extension ',[I,MN,E])).

/* CHECK ALL CONSTRAINTS */
run:-noDuplicateClassNames,noDuplicateMethodNames,illegalExt,methodBelongsToNonExistantClass.
