
/* file violet.conform.pl */

/* no Interface constraint */

noInterface:-forall(violetInterface(_,N,_,_,_),isError('no interfaces permitted: ',N)).

/* uniqueNames CONSTRAINT: Classes have unique names constraint */

findPair(I1,I2,N1,N2):-ename(I1,N1),ename(I2,N2),@I1 @< @I2.

uniqueNames:-forall(findPair(I1,I2,N,N),isError('unique class name constraint violated: ',[I1,I2,N])).

/* Null Names Constraint: classes cannot have null names */

nullNames:-forall(ename(I1,''),isError('class or interface has null name ',I1)).

/* inh constraint -- no association with a Triangle can have anything other than '' nothing */

translate(I1,I2,N1,N2):-ename(I1,N1),ename(I2,N2).
findTwo(SubN,SupN):-violetAssociation(_,_,'TRIANGLE',_,_,A,_,_,Sup,Sub),not(A = ''),translate(Sub,Sup,SubN,SupN).
findTwo(SubN,SupN):-violetAssociation(_,_,A,_,_,'TRIANGLE',_,_,Sub,Sup),not(A = ''),translate(Sub,Sup,SubN,SupN).

badInheritance :- forall(findTwo(E1,E2),isError('inheritance paired with non-null end ',[E1,E2])).

/* another inheritance constraint -- no labels! */

nonblank(R1,R2):-not(R1='');not(R2='').
ilabel(SubN,SupN):-violetAssociation(_,R1,'TRIANGLE',_,R2,_,_,_,Sup,Sub),nonblank(R1,R2),translate(Sub,Sup,SubN,SupN).
ilabel(SubN,SupN):-violetAssociation(_,R2,_,_,R1,'TRIANGLE',_,_,Sup,Sub),nonblank(R1,R2),translate(Sub,Sup,SubN,SupN).

noLabels:-forall(ilabel(A,B),isError('remove labels for inheritance ',[A,B])).

/* circular CONSTRAINT: Check circularity of class Hierarchy */

/*  utility predicates */
isClass(I,N):-violetClass(I,N,_,_,_,_).
ename(I,N):-isClass(I,N).

subClass(Sub,Sup):-violetAssociation(_,_,'TRIANGLE',_,_,_,_,_,Sup,Sub),isClass(Sup,_),isClass(Sub,_).
subClass(Sub,Sup):-violetAssociation(_,_,_,_,_,'TRIANGLE',_,_,Sub,Sup),isClass(Sup,_),isClass(Sub,_).
subOf(Sub,Sup):-subClass(Sub,Sup).
subOf(Sub,Sup):-subClass(Sub,X),subOf(X,Sup).

testCycle(N):-subOf(I,I),!,isClass(I,N).

circular:-forall(testCycle(N),isError('inheritance cycle has class ',N)).


%% CONSTRAINT -- Dotted inheritance only exists between classes and interfaces

dotTest(I) :- violetAssociation(_,_,_,'interfacenode',_,'TRIANGLE','classnode','DOTTED',N,_),ename(N,I).
dotTest(I) :- violetAssociation(_,_,'TRIANGLE','classnode',_,_,'interfacenode','DOTTED',_,N),ename(N,I).

dotCheck :- forall( dotTest(N),isError('interface X cannot implement class where X=',N)).

/* ALL CONSTRAINTS */

run:-noInterface,uniqueNames,nullNames,badInheritance,noLabels,
     circular, dotCheck.
