/* file: sdb2yuml.pl */

/* below are the joins that simplify writing VM2T code */

% here's an oddball case.  Suppose a SDB class tuple
% has a field which is quoted, as in '"abc"'.  This arises
% ONLY in the case of bootstrapping.  In such cases,
% double quotes must be replaced by some other character,
% which I arbitrarily decided would be %. 
% Why?  because yuml is broken.  It doesn't understand
% double quotes.  It can understand %.
% So... the fix is to replace all fields with double quotes in SDB 
% with fields in ypl with %.  This is what the xlate predicate does

replace(_, _, [], []).
replace(O, R, [O|T], [R|T2]) :- replace(O, R, T, T2).
replace(O, R, [H|T], [H|T2]) :- H \= O, replace(O, R, T, T2).

xlate(A,B):-atom_chars(A,L),replace('"','%',L,L1),atom_chars(B,L1).

%
%
% left arrow types (lat)
lat(arrow,'<').
lat(agg,'<>').
lat(none,'').
lat(comp,'++').
lat(gen,'^').
lat('','').

%right arrow types (rat)
rat(arrow,'>').
rat(agg,'<>').
rat(none,'').
rat(comp,'++').
rat(gen,'^').
rat('','').

yumlClass(I,N,F,M):-class(I,N,F1,M,_),xlate(F1,F).
yumlInterface(I,N,M):-interface(I,N,M).

yumlAssociation(Id,N1,R1,A1,N2,R2,A2):-association(Id,R1,AA1,R2,AA2,I1,I2),not(AA1==inherit),not(AA2==inherit),
        class(I1,N1,_,_,_),class(I2,N2,_,_,_),lat(AA1,A1),rat(AA2,A2).

yumlAssociation(ID,IN,'','^',CN,'',''):-classImplements(_,C,I),class(C,CN,_,_,_),interface(I,IN,_),getId(ID).
yumlAssociation(ID,Nx,'','',Nb,'','^'):-interfaceExtends(_,Ib,Ix),interface(Ib,Nb,_),interface(Ix,Nx,_),getId(ID).
yumlAssociation(ID,N2,'','^',N1,'',''):-class(_,N1,_,_,S),class(S,N2,_,_,_),getId(ID).

% I added this to remove null identifiers in earlier releases
:-dynamic total/1.

total(0).

getId(I):-total(C),retract(total(C)),A is C+1,assert(total(A)),atom_concat('i',A,I).