/* file violettpl.conform.pl */

/* uniqueNames CONSTRAINT: Classes and Interfaces have unique names constraint */

findPair(I1,I2,N1,N2):-ename(I1,N1),ename(I2,N2),@I1 @< @I2.
uniqueNames:-forall(findPair(I1,I2,N,N),isError('unique names constraint violated: ',[I1,I2,N])).

/* Null Names Constraint: classes and interfaces cannot have null names */

nullNames:-forall(ename(I1,''),isError('class or interface has null name ',I1)).

/* inh constraint -- no association with a Triangle can have anything other than '' nothing */

translate(I1,I2,N1,N2):-ename(I1,N1),ename(I2,N2).
findTwo(SubN,SupN):-violetTAssociation(_,_,'TRIANGLE',_,_,A,_,_,Sup,Sub),not(A = ''),translate(Sub,Sup,SubN,SupN).
findTwo(SubN,SupN):-violetTAssociation(_,_,A,_,_,'TRIANGLE',_,_,Sub,Sup),not(A = ''),translate(Sub,Sup,SubN,SupN).

badInheritance :- forall(findTwo(E1,E2),isError('inheritance paired with non-null end ',[E1,E2])).

/* another inheritance constraint -- no labels! */

nonblank(R1,R2):-not(R1='');not(R2='').
ilabel(SubN,SupN):-violetTAssociation(_,R1,'TRIANGLE',_,R2,_,_,_,Sup,Sub),nonblank(R1,R2),translate(Sub,Sup,SubN,SupN).
ilabel(SubN,SupN):-violetTAssociation(_,R2,_,_,R1,'TRIANGLE',_,_,Sup,Sub),nonblank(R1,R2),translate(Sub,Sup,SubN,SupN).

noLabels:-forall(ilabel(A,B),isError('remove labels for inheritance ',[A,B])).

/* circular CONSTRAINT: Check circularity of class Hierarchy */

/*  utility predicates */
isClass(I,N):-violetTClass(I,N,_,_,_,_).
isIntf(I,N):-violetTInterface(I,N,_,_,_).
ename(I,N):-isIntf(I,N);isClass(I,N).

subClass(Sub,Sup):-violetTAssociation(_,_,'TRIANGLE',_,_,_,_,_,Sup,Sub),isClass(Sup,_),isClass(Sub,_).
subClass(Sub,Sup):-violetTAssociation(_,_,_,_,_,'TRIANGLE',_,_,Sub,Sup),isClass(Sup,_),isClass(Sub,_).
subOf(Sub,Sup):-subClass(Sub,Sup).
subOf(Sub,Sup):-subClass(Sub,X),subOf(X,Sup).

testCycle(N):-subOf(I,I),!,isClass(I,N).

circular:-forall(testCycle(N),isError('class inheritance cycle has ',N)).

/* icircular CONSTRAINT: Check circularity of inheritance Hierarchy  */

subIntf(Sub,Sup):-violetTAssociation(_,_,'TRIANGLE',_,_,_,_,_,Sup,Sub),isIntf(Sup,_),isIntf(Sub,_).
subIntf(Sub,Sup):-violetTAssociation(_,_,_,_,_,'TRIANGLE',_,_,Sub,Sup),isIntf(Sup,_),isIntf(Sub,_).
isubOf(Sub,Sup):-subIntf(Sub,Sup).
isubOf(Sub,Sup):-subIntf(Sub,X),isubOf(X,Sup).

itestCycle(N):-isubOf(I,I),!,isIntf(I,N).

icircular:-forall(itestCycle(N),isError('interface inheritance cycle has ',N)).

/*
% noDoubleEnds CONSTRAINT: Check that no edge has diamond, arrow, triangle, etc on both sides 

hasEndShape('BLACK_DIAMOND').
hasEndShape('TRIANGLE').
%hasEndShape('V').
hasEndShape('DIAMOND').

testHasDoubleEnds(I,N):- violetTAssociation(_,I,_,Arrow1,_,ToNodeId,_,Arrow2,_,_),
                            hasEndShape(Arrow1), hasEndShape(Arrow2),
                                violetTClass(ToNodeId,ToNodeName,_,_,_,_),
                                    isError('double ended edge: ',[N,ToNodeName]).
testHasDoubleEnds(_,_).

noDoubleEnds :- forall(isClass(I,N),testHasDoubleEnds(I,N)).
inoDoubleEnds :- forall(isIntf(I,N),testHasDoubleEnds(I,N)).
*/

/*
%legalMultiplicity CONSTRAINT: no inheritance association can have multiplicies 

noMult('').

isTriangle('TRIANGLE').

testMultiplicity(I,N):- violetTAssociation(_,I1,Mult1,Arrow1,_,I2,Mult2,Arrow2,_,_),
                            (not(noMult(Mult1));not(noMult(Mult2))),
                                (isTriangle(Arrow1);isTriangle(Arrow2)), 
                                    isError('no roles allowed: ',[I1,I2]).
testMultiplicity(_,_).

legalMultiplicity :- forall(isClass(I,N),testMultiplicity(I,N)).
ilegalMultiplicity :- forall(isIntf(I,N),testMultiplicity(I,N)).
*/
/*
% here's an odd one: DIAMOND cannot be paired with '*' -- we can only approximate this
% check because roles can have cardinalities and just about anything else listed

aDiamond('DIAMOND').
aDiamond('BLACK_DIAMOND').

dtest(N) :- violetTAssociation(_,I1,'*',D,_,I2,_,_,_,_),aDiamond(D),ename(I1,N1),ename(I2,N2),N=[N1,N2].
dtest(N) :- violetTAssociation(_,I1,_,_,_,I2,'*',D,_,_),aDiamond(D),ename(I1,N1),ename(I2,N2),N=[N1,N2].

diamondCheck :- forall(dtest(N),isError('diamond * multiplicity illegal ',N)).
*/

/*
%% CONSTRAINT -- an inheritance association can have no multiplicities and no other arrowtypes

blank('').

allBlanks(R1,R2,A):- blank(R1),blank(R2),blank(A).

itest(N) :- violetTAssociation(C1,R1,'TRIANGLE',C2,R2,A,_,_,_,_),not(allBlanks(R1,R2,A)),ename(C1,N1),ename(C2,N2),N=[N1,N2].
itest(N) :- violetTAssociation(C2,R2,A,C1,R1,'TRIANGLE',_,_,_,_),not(allBlanks(R1,R2,A)),ename(C1,N1),ename(C2,N2),N=[N1,N2].

iCheck :- forall(itest(N),isError('inheritance can have no roles or other association types ',N)).
*/


%% CONSTRAINT -- Dotted inheritance only exists between classes and interfaces

dotTest(I) :- violetTAssociation(_,_,_,'interfacenode',_,'TRIANGLE','classnode','DOTTED',N,_),ename(N,I).
dotTest(I) :- violetTAssociation(_,_,'TRIANGLE','classnode',_,_,'interfacenode','DOTTED',_,N),ename(N,I).
dotTest(I) :- violetTAssociation(_,_,_,'classnode',_,'TRIANGLE','classnode','DOTTED',N,_),ename(N,I).
dotTest(I) :- violetTAssociation(_,_,'TRIANGLE','classnode',_,_,'classnode','DOTTED',_,N),ename(N,I).

dotCheck :- forall( dotTest(N),isError('interface X cannot implement class where X=',N)).

%% another odd constraint -- violet permits associations with middle labels.  Only violet, to my knowledge
%% has this "feature", and using it just causes problems.  Alert designers to errors if it is used.

middleLabelError:-forall( violetTMiddleLabels(_,A,B,C),isError('do not use middle label in association ',[A,B,C])).

/* ALL CONSTRAINTS */

run:-uniqueNames,nullNames,noLabels,circular, icircular,
            dotCheck, middleLabelError, badInheritance.
